package dev.fitko.fitconnect.cli;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.client.SenderClient;
import dev.fitko.fitconnect.client.SubscriberClient;
import dev.fitko.fitconnect.client.bootstrap.ApplicationConfigLoader;
import dev.fitko.fitconnect.client.bootstrap.ClientFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.file.Path;
import java.util.function.Supplier;
import java.util.stream.Collectors;

public final class CommandLineRunner {
    private static final Logger LOGGER = LoggerFactory.getLogger(CommandLineRunner.class);
    private static final String LOGO = "/splash_screen_banner.txt";

    private static final String DEFAULT_CONFIG_NAME = "config.yml";


    private CommandLineRunner() {
    }

    public static void main(final String[] args) {
        printSplashScreen();
        getCommandLineClient().run(args);
    }

    private static CommandLineClient getCommandLineClient() {
        final Supplier<SenderClient> senderClientSupplier = () -> ClientFactory.getSenderClient(loadConfig());
        final Supplier<SubscriberClient> subscriberClientSupplier = () -> ClientFactory.getSubscriberClient(loadConfig());
        return new CommandLineClient(senderClientSupplier, subscriberClientSupplier);
    }

    private static ApplicationConfig loadConfig() {
        try {
            return ApplicationConfigLoader.loadConfigFromEnvironment();
        } catch (final Exception e) {
            LOGGER.info("Loading config from default location {}", DEFAULT_CONFIG_NAME);
            return ApplicationConfigLoader.loadConfigFromPath(Path.of(DEFAULT_CONFIG_NAME));
        }
    }

    private static void printSplashScreen() {
        LOGGER.info("{}", getSplashScreenResource());
    }

    private static String getSplashScreenResource() {
        final InputStream is = CommandLineRunner.class.getResourceAsStream(LOGO);
        return new BufferedReader(new InputStreamReader(is)).lines().collect(Collectors.joining("\n"));
    }
}
